
import * as StorageConstants from "./storage-constants.js";
import {IndexedDbResponse} from "./indexedDb-response.js";
import {LocalStoreBase} from "./local-store-base.js";

export class MigratorBase extends LocalStoreBase {
    
    constructor() {
        super();
        this._migrations = [];
    }
    
    async migrateAsync(db, version) {
        // NOTE JB: One cannot insert data into the DB while a version change transaction is running. 
        //  - Thus all migrations needs to be completed before the object stores can be accessed.
        
        const response = new IndexedDbResponse();
        if (version < this._migrations.length) {
            
            // Run migrateAsync the data base to the requested version
            for (let i = version; i < this._migrations.length; i++) {
                const migrationStep = this._migrations[i];
                
                const handler = migrationStep.bind(this);
                const migrationStepResponse = await handler(db);
                if (!migrationStepResponse.isValid()) {
                    response.notification.addRange(migrationStepResponse.notification);
                    break;
                }
            }
            
        }
        else {
            response.notification.addError(StorageConstants.Errors.IndexedDbGeneral, `There is no migration for version '${version}'`);
        }

        return response;
    }
}